"""exfill will scrape LinkedIn job postings, parse out details about
each posting, then combine all of the information into a single useable
csv file.
"""
import argparse
import configparser
import logging
import sys
import os

from parsers.linkedin_parser import parse_linkedin_postings  # type: ignore
from scrapers.linkedin_scraper import scrape_linkedin_postings  # type: ignore


def init_parser() -> dict:
    """Initialize argument parser."""
    parser = argparse.ArgumentParser()
    parser.add_argument("site", choices=["linkedin"], help="Site to scrape")
    parser.add_argument(
        "action", choices=["scrape", "parse"], help="Action to perform"
    )
    return vars(parser.parse_args())


def load_config() -> configparser.ConfigParser:
    """Load config file"""

    config_file = os.path.dirname(__file__) + "/config.ini"
    if not os.path.exists(config_file):
        print(
            "Exiting app as the following config file does not exist: ",
            config_file,
            file=sys.stderr,
        )
        sys.exit()

    config = configparser.ConfigParser(
        interpolation=configparser.ExtendedInterpolation()
    )
    config.read(config_file)

    return config


def create_dirs(config) -> None:
    """Create directories referenced in the config file"""
    # for item in config['Directories']:
    for dir_path in config.items("Directories"):
        if not os.path.exists(dir_path[1]):
            os.mkdir(dir_path[1])


def main() -> None:
    """Main controller function used to call child functions/modules."""
    # Load config
    config = load_config()

    create_dirs(config)

    # Initialize logging
    log_file_name = config["Paths"]["app_log"]
    logging.basicConfig(
        filename=log_file_name,
        level=logging.INFO,  # level=logging.INFO should be default
        format="[%(asctime)s] [%(levelname)s] - %(message)s",
        filemode="w+",
    )

    args = init_parser()
    logging.info("Starting app with the following input args: %s", args)

    if args["site"] == "linkedin":
        if args["action"] == "scrape":
            # postings_to_scrape will round up by 25 as 25
            # postings are loaded per page
            scrape_linkedin_postings(config, postings_to_scrape=5)
        if args["action"] == "parse":
            parse_linkedin_postings(config)

    logging.info("Finished execution.  Exiting application.")
    sys.exit()


if __name__ == "__main__":

    main()



"""Parser module will process and aggregate job posting files.
"""

from .parser import Parser


def parse_linkedin_postings(config):
    """Main parser function that controls program flow"""

    linkedin_parser = Parser(config)

    linkedin_parser.parse_files()

    linkedin_parser.export()



"""Scraper module will open a FireFox browser and scrape html
for job postings on LinkedIn.
"""
import datetime
import json
import logging
import os
import re
import time

from bs4 import BeautifulSoup
from selenium import webdriver


def login(config, driver) -> None:
    """Log into LinkedIn"""
    logging.info("Reading in creds")
    with open(config["Paths"]["creds"], encoding="UTF-8") as creds:
        cred_dict = json.load(creds)["linkedin"]
    logging.info("User name - %s", cred_dict["username"])

    logging.info("Navigating to login page")
    url = "https://www.linkedin.com/login"
    driver.get(url)

    logging.info("Signing in")
    username_field = driver.find_element_by_id("username")
    username_field.send_keys(cred_dict["username"])

    password_field = driver.find_element_by_id("password")
    password_field.send_keys(cred_dict["password"])

    submit_button = driver.find_element_by_xpath(
        "//button[@aria-label='Sign in']"
    )
    submit_button.click()


def export_html(config, soup) -> None:
    """Export single posting to html file"""

    output_file_prefix = os.path.join(
        config["Scraper"]["linkedin_out_dir"], "jobid_"
    )

    # Find jobid - it's easier with beautifulsoup
    # Example:
    # <a ... href="/jobs/view/2963302086/?alternateChannel...">
    logging.info("Finding Job ID")
    posting_details = soup.find(class_="job-view-layout")
    anchor_link = posting_details.find("a")
    jobid_search = re.search(r"view/(\d*)/", anchor_link["href"])
    jobid = jobid_search.group(1)

    # File name syntax:
    # jobid_[JOBID]_[YYYYMMDD]_[HHMMSS].html
    # Example:
    # jobid_2886320758_20220322_120555.html
    time_stamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    output_file = output_file_prefix + jobid + "_" + time_stamp + ".html"

    logging.info("Exporting jobid %s to %s", jobid, output_file)
    with open(output_file, "w+", encoding="UTF-8") as file:
        file.write(posting_details.prettify())


def scrape_linkedin_postings(config, postings_to_scrape: int) -> None:
    """Main scraper function that controls program flow"""
    logging.info("Scraping linkedin")

    logging.info("Initalizing browser")
    driver = webdriver.Firefox(
        executable_path=os.path.dirname(os.path.dirname(__file__))
        + "/"
        + config["Paths"]["gecko_driver"],
        service_log_path=config["Paths"]["gecko_log"],
    )

    driver.implicitly_wait(10)
    driver.set_window_size(1800, 600)

    login(config, driver)

    postings_scraped_total = 0

    while postings_scraped_total < postings_to_scrape:
        # required to prevent server timeout
        time.sleep(2)
        url = (
            "https://www.linkedin.com/jobs/search"
            + "?keywords=devops&location=United%20States&f_WT=2&&start="
            + str(postings_scraped_total)
        )
        logging.info("Loading url: %s", url)
        driver.get(url)

        # There are 25 postings per page.  Postings are loaded dynamically
        # so they cannot all be loaded and iterated at once
        postings_scraped_page = 0
        while postings_scraped_page < 25:

            # required to prevent server timeout
            time.sleep(2)
            logging.info("START - Process new posting")
            logging.info("Updating card anchor list")
            # Create a list of each card (list of anchor tags).  Example card below
            # <a href="/jobs/view/..." id="ember310" class="disabled ember-view
            # job-card-container__link job-card-list__title"> blah </a>
            card_anchor_list = driver.find_elements_by_class_name(
                "job-card-list__title"
            )
            # About 7 are loaded initially.  More are loaded dynamically as the user
            # scrolls down
            card_anchor_list_count = len(card_anchor_list)
            logging.info("Anchor list count - %s", card_anchor_list_count)

            # Scroll to the next element using javascript
            logging.info("Scrolling to - %s", postings_scraped_total)
            driver.execute_script(
                "arguments[0].scrollIntoView(true);",
                card_anchor_list[postings_scraped_total],
            )
            logging.info("Clicking on %s", postings_scraped_total)
            card_anchor_list[postings_scraped_total].click()
            time.sleep(2)  # hopefully helps with missing content

            # Initialize beautifulsoup (used for simple exporting)
            soup = BeautifulSoup(driver.page_source, "html.parser")
            export_html(config, soup)

            logging.info("END - Process new posting")

            postings_scraped_total += 1
            postings_scraped_page += 1
    logging.info("Closing browser")
    driver.close()
